SystemOrganization addCategory: #'Xpf-CodeTalkLegacy'!

!DataStream methodsFor: '*xpf-codetalklegacy' stamp: 'mt 3/24/2009 15:43'!
references

    ^ Dictionary new! !

!ByteArray class methodsFor: '*xpf-codetalklegacy' stamp: 'mt 5/3/2009 21:24'!
fromHex: hexString 
	"Returns a byteArray from a hex string. The hex string can have
	leading zeros that will not be truncated, preserving the size of the
	byteArray for things like cyrptograpy"
	| anInteger hexSize stringSize aStringOfHex |
	aStringOfHex := (hexString copyWithoutAll: (Array with: $  with: Character cr with: Character tab)).
	stringSize := aStringOfHex size.
	(stringSize > 3
			and: [(aStringOfHex copyFrom: 1 to: 3) = '16r'])
		ifTrue: [anInteger := Number readFrom: (aStringOfHex asUppercase at: 3 put: $r;
							 yourself).
			hexSize := aStringOfHex size - 3 / 2]
		ifFalse: [anInteger := Number readFrom: aStringOfHex asUppercase base: 16.
			hexSize := aStringOfHex size / 2].
	^ anInteger asByteArrayOfSize2: hexSize asInteger! !

!MCMethodDefinition class methodsFor: '*xpf-codetalklegacy-override' stamp: 'mt 1/22/2010 13:51:00'!
forMethodReference: aMethodReference
	"CodeTalk: Get the markup from the method reference and put it into the definition."

	| definition |
	definition := self cachedDefinitions at: aMethodReference compiledMethod ifAbsent: [].
	(definition isNil
		or: [definition selector ~= aMethodReference methodSymbol]
		or: [definition className ~= aMethodReference classSymbol]
		or: [definition classIsMeta ~= aMethodReference classIsMeta]
		or: [definition category ~= aMethodReference category])
			ifTrue: [definition := (self 
						className: aMethodReference classSymbol
						classIsMeta: aMethodReference classIsMeta
						selector: aMethodReference methodSymbol
						category: aMethodReference category
						timeStamp: aMethodReference timeStamp
						source: aMethodReference source)
						sourceText: nil.
					self cachedDefinitions at: aMethodReference compiledMethod put: definition].
	^ definition
	! !

!MCMethodDefinition methodsFor: '*xpf-codetalklegacy-override' stamp: 'cs 4/17/2009 18:23'!
addMethodAdditionTo: aCollection
	| methodAddition |
	methodAddition := MethodAddition new
		compile: self sourceText
		classified: category
		withStamp: timeStamp
		notifying: (SyntaxError new category: category)
		logSource: true
		inClass: self actualClass.
	"This might raise an exception and never return"
	methodAddition createCompiledMethod.
	aCollection add: methodAddition.
! !

!MCMethodDefinition methodsFor: '*xpf-codetalklegacy-override' stamp: 'mt 6/12/2009 14:19'!
load

	self actualClass
		compile: self sourceText
		classified: category
		withStamp: timeStamp
		notifying: (SyntaxError new category: category)! !

!MCMethodDefinition methodsFor: '*xpf-codetalklegacy' stamp: 'cs 4/20/2009 16:16'!
sourceText

	^ sourceText ifNil: [self source asText]! !

!MCMethodDefinition methodsFor: '*xpf-codetalklegacy' stamp: 'mt 3/24/2009 15:52'!
sourceText: aText

	sourceText := aText.! !

Object subclass: #XpfCodeTalkLegacy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalkLegacy'!

!XpfCodeTalkLegacy class methodsFor: 'initialization' stamp: 'mt 4/29/2010 10:07:54.048'!
initialize
	"Resend this message whenever you checkout a new version of CodeTalk."
	
	super initialize.
	self
		initializeMonticello;
		initializeOtherClasses.! !

!XpfCodeTalkLegacy class methodsFor: 'initialization' stamp: 'mt 4/29/2010 10:07:00.298'!
initializeMonticello

	(MCMethodDefinition allInstVarNames includes: #sourceText)
		ifFalse:[MCMethodDefinition addInstVarName: #sourceText].
! !

!XpfCodeTalkLegacy class methodsFor: 'initialization' stamp: 'mt 4/29/2010 10:07:28.149'!
initializeOtherClasses
	
	(Smalltalk includesKey: #XpfComplexAttribute)
		ifTrue: [XpfComplexAttribute reset].! !

XpfCtMorphAttribute subclass: #XpfComplexAttribute
	instanceVariableNames: 'tag chat chatMorph deleteMorph resizeMorph morph creator timestamp comment commentMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalkLegacy'!

XpfComplexAttribute class
	instanceVariableNames: 'colorTable readCache'!

XpfComplexAttribute class
	instanceVariableNames: 'colorTable readCache'!

!XpfComplexAttribute class methodsFor: 'color mapping' stamp: 'mt 5/28/2009 19:15'!
colorForTag: aSymbol

	colorTable ifNil: [colorTable := Dictionary newFrom: {
		#bad -> (Color red alpha: 0.4).
		#talk -> (Color yellow alpha: 0.4).
		#comment -> (Color green alpha: 0.4)}].
	
	^ colorTable at: aSymbol ifAbsent: [Color transparent]! !

!XpfComplexAttribute class methodsFor: 'instance creation' stamp: 'mt 5/2/2009 16:11'!
forTag: aSymbol

	^ self new
		tag: aSymbol;
		timestamp: TimeStamp now;
		creator: Utilities authorInitials! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 6/23/2009 17:22'!
read: aComplexAttribute
	"Tell the local cache that the given attribute was read by the developer."
	
	(Preferences
		preferenceAt: #rememberMarkupReads
		ifAbsent: [Preference new rawValue: false])
		preferenceValue
			ifFalse: [^ self].
	
	(readCache ifNil: [readCache := Set new])
		add: (self uniqueArrayFor: aComplexAttribute).! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 6/2/2009 17:59'!
reset
	"Free all cached values."
	
	colorTable := nil.
	readCache := nil.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/3/2009 21:19'!
scanChatFrom: strm for: attribute

	| chatCopy |
	chatCopy := Json readFrom: (ByteArray fromHex: (strm upTo: $-)) asString readStream.
	chatCopy do: [:line |
		line at: #timestamp put: (TimeStamp fromString: (line at: #timestamp))].
	attribute chat: chatCopy asOrderedCollection.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/28/2009 19:10'!
scanCommentFrom: strm for: attribute

	attribute comment: (ByteArray fromHex: (strm upTo: $-)) asString.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/3/2009 21:19'!
scanCreatorFrom: strm for: attribute

	attribute creator: (ByteArray fromHex: (strm upTo: $-)) asString.! !

!XpfComplexAttribute class methodsFor: 'instance creation' stamp: 'mt 10/16/2009 18:50'!
scanFrom: strm

	| count instance substream content |
	instance := self new.
	content := strm upTo: $#.
	count := (content findTokens: $-) size.

	(count < 4 or: [count > 5])
		ifTrue: [Transcript show: '[CodeTalk] Parsing failed.'; cr. ^ instance].
		
	substream := content readStream.
	self
		scanTimestampFrom: substream for: instance;
		scanCreatorFrom: substream for: instance;
		scanTagFrom: substream for: instance;
		scanChatFrom: substream for: instance.
	
	count = 5 ifTrue: [self scanCommentFrom: substream for: instance].
	instance creator.
	^ instance! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/1/2009 14:37'!
scanTagFrom: strm for: attribute

	attribute tag: (strm upTo: $-) asSymbol.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/3/2009 21:19'!
scanTimestampFrom: strm for: attribute

	attribute timestamp: (TimeStamp fromString:
		(ByteArray fromHex: (strm upTo: $-)) asString).! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 5/28/2009 19:14'!
tags
	"Return a list of all available tags."

	^ #(bad talk comment)! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 6/23/2009 17:22'!
uniqueArrayFor: aComplexAttribute
	
	^ {	aComplexAttribute creator.
		aComplexAttribute timestamp printString.
		aComplexAttribute chat isEmpty
			ifTrue: ['']
			ifFalse: [aComplexAttribute chat last at: #author].
		aComplexAttribute chat isEmpty
			ifTrue: ['']
			ifFalse: [(aComplexAttribute chat last at: #timestamp) printString].
		aComplexAttribute tag.
		aComplexAttribute comment}.! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 6/23/2009 17:22'!
wasRead: aComplexAttribute
	"Did the developer already read the given attribute?"
	
	(Preferences
		preferenceAt: #rememberMarkupReads
		ifAbsent: [Preference new rawValue: false])
		preferenceValue
			ifFalse: [readCache := nil. ^ false].
	
	^ (readCache ifNil: [readCache := Set new])
		includes: (self uniqueArrayFor: aComplexAttribute)! !

!XpfComplexAttribute methodsFor: 'chat' stamp: 'mt 7/2/2009 22:41'!
addChatLine: aString

	self chat add: (Dictionary newFrom: {
		#author -> Utilities authorInitials.
		#timestamp -> TimeStamp now.
		#message -> aString withBlanksTrimmed}).
	self changed: #chatLineAdded.! !

!XpfComplexAttribute methodsFor: 'chat' stamp: 'mt 4/30/2009 17:08'!
addDependent: anObject

	(self dependents includes: anObject)
		ifTrue: [^ self].
		
	^ super addDependent: anObject.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 4/29/2009 21:51'!
chat

	chat ifNil: [chat := OrderedCollection new].
	^ chat! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 4/30/2009 11:15'!
chat: aCollection

	chat := aCollection.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/1/2009 17:11'!
chatMorph

	chatMorph ifNil: [chatMorph := XpfCtChatIconMorph new attribute: self].
	^ chatMorph! !

!XpfComplexAttribute methodsFor: 'reading' stamp: 'mt 6/8/2009 11:01'!
codeSnippetFor: aText
	"Return the code snippet that is marked by this attribute."
	
	(aText find: self)
		ifNotNilDo: [:interval |
			^ (aText copyFrom: interval first to: interval last) asString withBlanksTrimmed].
	^ ''! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/1/2009 14:31'!
color

	^ self class colorForTag: self tag! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/28/2009 19:09'!
comment

	comment ifNil: [comment := ''].
	^ comment! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 6/8/2009 10:36'!
comment: aString

	comment := aString asString withBlanksTrimmed.
	self changed: #commentChanged.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/28/2009 19:07'!
commentMorph

	commentMorph ifNil: [commentMorph := XpfCtCommentIconMorph new attribute: self].
	^ commentMorph! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 10/16/2009 18:54'!
creator

	creator ifNil: [creator := 'n/a'. Transcript show: '[CodeTalk] Attribute without author detected!!'; cr].
	^ creator! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 10/16/2009 18:47'!
creator: authorInitials

	creator := authorInitials.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/1/2009 17:24'!
deleteMorph

	deleteMorph ifNil: [deleteMorph := XpfCtDeleteIconMorph new attribute: self].
	^ deleteMorph! !

!XpfComplexAttribute methodsFor: 'displaying' stamp: 'mt 5/1/2009 15:37'!
displayOn: aCanvas in: drawBounds
	"Show some stuff on the canvas for this attribute."
	
	aCanvas
		fillRectangle: drawBounds
		color: self color.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'cs 5/4/2009 02:27'!
includesChatLine: aString

	self chat do: [:dict |
		((dict at: #message) includesSubString: aString)
			ifTrue: [^ true]].
	^ false! !

!XpfComplexAttribute methodsFor: 'chat' stamp: 'mt 6/12/2009 15:48'!
isAuthorMentioned
	"Look into the chat messages to find out if this image's author is mentioned in a message."
	
	| rx |
	rx := ('\\<{1}\\>' format: {Utilities authorInitials}) asRegex.
	^ self chat anySatisfy: [:line | rx search: (line at: #message)]! !

!XpfComplexAttribute methodsFor: 'testing' stamp: 'mt 6/5/2009 14:41'!
isMine

	^ self creator = Utilities authorInitials! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/28/2009 19:18'!
morph

	morph ifNil: [morph := XpfCtRowMorph for: {
		self tag = #comment
			ifTrue: [self commentMorph]
			ifFalse: [self chatMorph].
		self resizeMorph.
		self deleteMorph}].
	^ morph! !

!XpfComplexAttribute methodsFor: 'reading' stamp: 'mt 6/2/2009 18:04'!
read

	self class read: self.! !

!XpfComplexAttribute methodsFor: 'event handling' stamp: 'mt 5/1/2009 17:33'!
requestDeletion

	self changed: #deleteRequested with: self.! !

!XpfComplexAttribute methodsFor: 'event handling' stamp: 'mt 5/3/2009 19:47'!
requestResizing
	"No need to give us away because the pure selection in a paragraph will handle this."

	self changed: #resizeRequested.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/18/2009 10:40'!
resetMorphs

	morph := nil.
	chatMorph := nil.
	resizeMorph := nil.
	deleteMorph := nil.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/3/2009 19:20'!
resizeMorph

	resizeMorph ifNil: [resizeMorph := XpfCtResizeIconMorph new attribute: self].
	^ resizeMorph! !

!XpfComplexAttribute methodsFor: 'styling' stamp: 'mt 3/19/2010 10:14:04.782'!
shoutShouldPreserve

	^ false! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/1/2009 14:31'!
tag

	^ tag! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/18/2009 10:42'!
tag: aSymbol
	"Tag this attribute. Different tags allows to display this attribute in different ways."

	tag := (self class tags includes: aSymbol) ifTrue: [aSymbol] ifFalse: [].
	self resetMorphs.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:47'!
timestamp

	timestamp ifNil: [timestamp := TimeStamp now].
	^ timestamp! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:45'!
timestamp: aTimeStamp

	timestamp := aTimeStamp.! !

!XpfComplexAttribute methodsFor: 'displaying' stamp: 'mt 1/16/2010 17:11'!
updateMorphs: someAttributes start: startIndex stop: stopIndex

	((someAttributes size = 1) and: [startIndex ~= stopIndex])
		ifTrue: [self resizeMorph visible: true]
		ifFalse: [self resizeMorph visible: false]! !

!XpfComplexAttribute methodsFor: 'reading' stamp: 'mt 6/2/2009 18:04'!
wasRead

	^ self class wasRead: self! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 15:51'!
writeChatOn: strm

	| chatCopy |
	chatCopy := self chat collect: [:line |
		Dictionary newFrom: {
			#timestamp -> (line at: #timestamp) printString.
			#author -> (line at: #author).
			#message -> (line at: #message)}].
	strm nextPutAll: (Json render: chatCopy) toHex.
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/28/2009 20:08'!
writeCommentOn: strm

	strm nextPutAll: self comment toHex.
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 15:52'!
writeCreatorOn: strm

	strm nextPutAll: self creator toHex.
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 6/24/2009 09:27'!
writeScanOn: strm

	strm nextPut: $h.
	self
		writeTimestampOn: strm;
		writeCreatorOn: strm;
		writeTagOn: strm;
		writeChatOn: strm;
		writeCommentOn: strm.
	strm nextPut: $#. "mark end of attribute stream"! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/1/2009 14:34'!
writeTagOn: strm

	strm nextPutAll: (self tag ifNil: [$0]).
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 16:08'!
writeTimestampOn: strm

	strm nextPutAll: self timestamp printString toHex.
	strm nextPut: $-.! !

TestCase subclass: #XpfCtComplexAttributeTest
	instanceVariableNames: 'sut sut2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalkLegacy'!

!XpfCtComplexAttributeTest methodsFor: 'running' stamp: 'mt 5/28/2009 19:19'!
setUp

	super setUp.
	sut := XpfComplexAttribute forTag: #bad.
	sut
		addChatLine: 'She sells sea shells at the sea shore.';
		creator: Utilities authorInitials, 'test'.
	sut2 := XpfComplexAttribute forTag: #comment.
	sut2 comment: 'This is just a comment'.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 6/8/2009 11:03'!
testChatLineTrimmed

	sut addChatLine: 'Foobar.'.
	self should: (sut chat last at: #message) = 'Foobar.'.
	sut addChatLine: '  
	Foobar.
	  '.
	self should: (sut chat last at: #message) = 'Foobar.'.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 6/8/2009 10:58'!
testCodeSnippet
	"Try to mark something in Collection>>do:. Then get the code snippet of the markup."
	
	| source ref |
	ref := MethodReference class: Collection selector: #do:.
	source := ref sourceCode.
	source addAttribute: sut from: 1 to: 10.
	self should: (sut codeSnippetFor: source) = 'do: aBlock'.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 6/8/2009 11:03'!
testCodeSnippetTrimmed
	
	| source ref |
	ref := MethodReference class: Collection selector: #do:.
	source := ref sourceCode.
	source addAttribute: sut from: 11 to: 18.
	self should: (sut codeSnippetFor: source) = '"Eval'.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 6/8/2009 10:38'!
testCommentTrimmed
	"Remove all trailing or leading blanks from a comment when it is set."
	
	sut comment: 'foobar'.
	self should: sut comment = 'foobar'.
	sut comment: '  foobar  '.
	self should: sut comment = 'foobar'.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/28/2009 12:10'!
testInstanceCreation

	self
		should: sut tag = #bad;
		should: sut creator = (Utilities authorInitials, 'test');
		should: sut chat size = 1;
		should: (sut chat first at: #author) = Utilities authorInitials;
		should: sut timestamp <= (sut chat first at: #timestamp)! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 6/2/2009 17:59'!
testReading

	XpfComplexAttribute reset. "Clear read cache."
	self
		shouldnt: (XpfComplexAttribute wasRead: sut);
		shouldnt: (XpfComplexAttribute wasRead: sut2).
	XpfComplexAttribute read: sut.
	self
		should: (XpfComplexAttribute wasRead: sut);
		shouldnt: (XpfComplexAttribute wasRead: sut2).
	self
		shouldnt: [XpfComplexAttribute
			read: sut;
			read: sut] raise: Error.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 6/11/2009 23:32'!
testRegex
	
	self should: ('([A-Z]\w*)(\sclass)?>>#?(\w[\w:]*)' asRegex isKindOf: RxMatcher).! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/2/2009 15:35'!
testScanning

	| stream attr |
	stream := (String new: 500) writeStream.
	sut writeScanOn: stream.
	attr := XpfComplexAttribute scanFrom:
		(stream contents readStream position: 1). "Skip first 'h' to identify complex attribute. See RunArray class>>scanFrom:"
	
	self
		should: sut tag = attr tag;
		should: sut timestamp = attr timestamp;
		should: sut creator = attr creator;
		should: sut chat size = attr chat size;
		should: (sut chat first at: #author) = (attr chat first at: #author);
		should: (sut chat first at: #timestamp) = (attr chat first at: #timestamp);
		should: (sut chat first at: #message) = (attr chat first at: #message)! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/28/2009 19:22'!
testScanningComments

	| stream attr |
	stream := (String new: 500) writeStream.
	sut2 writeScanOn: stream.
	attr := XpfComplexAttribute scanFrom:
		(stream contents readStream position: 1). "Skip first 'h' to identify complex attribute. See RunArray class>>scanFrom:"
	
	self
		should: sut2 tag = attr tag;
		should: sut2 comment = attr comment.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/18/2009 10:31'!
testTagAcccessing

	self should: sut tag = #bad.
	sut tag: #'0'.
	self should: sut tag isNil.
	sut tag: #talk.
	self should: sut tag = #talk.
	sut tag: #shouldNotBeAllowed.
	self should: sut tag isNil.! !

!RunArray class methodsFor: '*xpf-codetalklegacy-override' stamp: 'mt 4/30/2009 11:37'!
scanFrom: strm
	"Read the style section of a fileOut or sources file.  nextChunk has already been done.  We need to return a RunArray of TextAttributes of various kinds.  These are written by the implementors of writeScanOn:"
	| rr vv aa this |
	(strm peekFor: $( ) ifFalse: [^ nil].
	rr := OrderedCollection new.
	[strm skipSeparators.
	 strm peekFor: $)] whileFalse: 
		[rr add: (Number readFrom: strm)].
	vv := OrderedCollection new.	"Value array"
	aa := OrderedCollection new.	"Attributes list"
	[(this := strm next) == nil] whileFalse: [
		this == $, ifTrue: [vv add: aa asArray.  aa := OrderedCollection new].
		this == $a ifTrue: [aa add: 
			(TextAlignment new alignment: (Integer readFrom: strm))].
		this == $f ifTrue: [aa add: 
			(TextFontChange new fontNumber: (Integer readFrom: strm))].
		this == $F ifTrue: [aa add: (TextFontReference toFont: 
			(StrikeFont familyName: (strm upTo: $#) size: (Integer readFrom: strm)))].
		this == $b ifTrue: [aa add: (TextEmphasis bold)].
		this == $i ifTrue: [aa add: (TextEmphasis italic)].
		this == $u ifTrue: [aa add: (TextEmphasis underlined)].
		this == $= ifTrue: [aa add: (TextEmphasis struckOut)].
		this == $n ifTrue: [aa add: (TextEmphasis normal)].
		this == $- ifTrue: [aa add: (TextKern kern: -1)].
		this == $+ ifTrue: [aa add: (TextKern kern: 1)].
		this == $c ifTrue: [aa add: (TextColor scanFrom: strm)]. "color"
		this == $h ifTrue: [aa add: (XpfComplexAttribute scanFrom: strm)]. "background color - CodeTalk"
		this == $L ifTrue: [aa add: (TextLink scanFrom: strm)].	"L not look like 1"
		this == $R ifTrue: [aa add: (TextURL scanFrom: strm)].
				"R capitalized so it can follow a number"
		this == $q ifTrue: [aa add: (TextSqkPageLink scanFrom: strm)].
		this == $p ifTrue: [aa add: (TextSqkProjectLink scanFrom: strm)].
		this == $P ifTrue: [aa add: (TextPrintIt scanFrom: strm)].
		this == $d ifTrue: [aa add: (TextDoIt scanFrom: strm)].
		"space, cr do nothing"
		].
	aa size > 0 ifTrue: [vv add: aa asArray].
	^ self runs: rr asArray values: vv asArray
"
RunArray scanFrom: (ReadStream on: '(14 50 312)f1,f1b,f1LInteger +;i')
"! !

!String methodsFor: '*xpf-codetalklegacy' stamp: 'mt 5/2/2009 15:52'!
toHex

	^ self
		inject: ''
		into: [:result :char | result, char hex]! !

!Integer methodsFor: '*xpf-codetalklegacy' stamp: 'mt 5/3/2009 21:24'!
asByteArrayOfSize2: aSize 
	"Answer a ByteArray of aSize with my value, most-significant byte first."
	| answer digitPos |
	aSize < self digitLength ifTrue: [ Error signal: 'number to large for byte array' ].
	answer := ByteArray new: aSize.
	digitPos := 1.
	aSize 
		to: aSize - self digitLength + 1
		by: -1
		do: 
			[ : pos | 
			answer 
				at: pos
				put: (self digitAt: digitPos).
			digitPos := digitPos + 1 ].
	^ answer! !
XpfCodeTalkLegacy initialize!
